@@ -202,7 +202,8 @@ static void InsertRow(ssize_t depth,unsigned char *p,ssize_t y,Image *image)
     case 8: /* Convert PseudoColor scanline. */
       {
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (q == (PixelPacket *) NULL) break;
+        if (q == (PixelPacket *) NULL)
+          break;
         indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (ssize_t) image->columns; x++)
         {
@@ -287,6 +288,15 @@ static int GetCutColors(Image *image)
 */
 static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define ThrowCUTReaderException(severity,tag) \
+{ \
+  if (palette != NULL) \
+    palette=DestroyImage(palette); \
+  if (clone_info != NULL) \
+    clone_info=DestroyImageInfo(clone_info); \
+  ThrowReaderException(severity,tag); \
+}
+
   Image *image,*palette;
   ImageInfo *clone_info;
   MagickBooleanType status;
@@ -308,12 +318,12 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -331,7 +341,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Header.Reserved=ReadBlobLSBShort(image);
 
   if (Header.Width==0 || Header.Height==0 || Header.Reserved!=0)
-    CUT_KO:  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    CUT_KO:  ThrowCUTReaderException(CorruptImageError,"ImproperImageHeader");
 
   /*---This code checks first line of image---*/
   EncodedByte=ReadBlobLSBShort(image);
@@ -344,7 +354,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if((int) RunCount<0x80) i=(ssize_t) RunCountMasked;
       offset=SeekBlob(image,TellBlob(image)+i,SEEK_SET);
       if (offset < 0)
-        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        ThrowCUTReaderException(CorruptImageError,"ImproperImageHeader");
       if(EOFBlob(image) != MagickFalse) goto CUT_KO;  /*wrong data*/
       EncodedByte-=i+1;
       ldblk+=(ssize_t) RunCountMasked;
@@ -371,6 +381,10 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (status == MagickFalse)
     {
       InheritException(exception,&image->exception);
+      if (palette != NULL)
+        palette=DestroyImage(palette);
+      if (clone_info != NULL)
+        clone_info=DestroyImageInfo(clone_info);
       return(DestroyImageList(image));
     }
 
@@ -438,6 +452,8 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       PalHeader.MaxGreen=ReadBlobLSBShort(palette);
       PalHeader.MaxBlue=ReadBlobLSBShort(palette);
       (void) ReadBlob(palette,20,(unsigned char *) PalHeader.PaletteId);
+      if (EOFBlob(image))
+        ThrowCUTReaderException(CorruptImageError,"UnexpectedEndOfFile");
 
       if(PalHeader.MaxIndex<1) goto ErasePalette;
       image->colors=PalHeader.MaxIndex+1;
@@ -478,6 +494,8 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
 
         }
+      if (EOFBlob(image))
+        ThrowCUTReaderException(CorruptImageError,"UnexpectedEndOfFile");
     }
 
 
@@ -490,7 +508,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (AcquireImageColormap(image,image->colors) == MagickFalse)
         {
         NoMemory:
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          ThrowCUTReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
 
       for (i=0; i < (ssize_t)image->colors; i++)
@@ -541,7 +559,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if((int) RunCount>0x80)
             {
               RunValue=(unsigned char) ReadBlobByte(image);
-              (void) ResetMagickMemory(ptrB,(int) RunValue,(size_t) RunCountMasked);
+              (void) memset(ptrB,(int) RunValue,(size_t) RunCountMasked);
             }
           else {
             (void) ReadBlob(image,(size_t) RunCountMasked,ptrB);
