@@ -209,7 +209,7 @@ static LZWInfo *AcquireLZWInfo(Image *image,const size_t data_size)
   lzw_info=(LZWInfo *) AcquireMagickMemory(sizeof(*lzw_info));
   if (lzw_info == (LZWInfo *) NULL)
     return((LZWInfo *) NULL);
-  (void) ResetMagickMemory(lzw_info,0,sizeof(*lzw_info));
+  (void) memset(lzw_info,0,sizeof(*lzw_info));
   lzw_info->image=image;
   lzw_info->data_size=data_size;
   one=1;
@@ -226,9 +226,9 @@ static LZWInfo *AcquireLZWInfo(Image *image,const size_t data_size)
       lzw_info=RelinquishLZWInfo(lzw_info);
       return((LZWInfo *) NULL);
     }
-  (void) ResetMagickMemory(lzw_info->table[0],0,MaximumLZWCode*
+  (void) memset(lzw_info->table[0],0,MaximumLZWCode*
     sizeof(**lzw_info->table));
-  (void) ResetMagickMemory(lzw_info->table[1],0,MaximumLZWCode*
+  (void) memset(lzw_info->table[1],0,MaximumLZWCode*
     sizeof(**lzw_info->table));
   for (i=0; i <= (ssize_t) lzw_info->maximum_data_value; i++)
   {
@@ -641,9 +641,9 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   /*
     Initialize GIF encoder.
   */
-  (void) ResetMagickMemory(hash_code,0,MaxHashTable*sizeof(*hash_code));
-  (void) ResetMagickMemory(hash_prefix,0,MaxHashTable*sizeof(*hash_prefix));
-  (void) ResetMagickMemory(hash_suffix,0,MaxHashTable*sizeof(*hash_suffix));
+  (void) memset(hash_code,0,MaxHashTable*sizeof(*hash_code));
+  (void) memset(hash_prefix,0,MaxHashTable*sizeof(*hash_prefix));
+  (void) memset(hash_suffix,0,MaxHashTable*sizeof(*hash_suffix));
   number_bits=data_size;
   max_code=MaxCode(number_bits);
   clear_code=((short) one << (data_size-1));
@@ -929,6 +929,12 @@ static ssize_t ReadBlobBlock(Image *image,unsigned char *data)
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static void *DestroyGIFProfile(void *profile)
+{
+  return((void *) DestroyStringInfo((StringInfo *) profile));
+}
+
 static MagickBooleanType PingGIFImage(Image *image)
 {
   unsigned char
@@ -960,13 +966,23 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define BitSet(byte,bit)  (((byte) & (bit)) == (bit))
 #define LSBFirstOrder(x,y)  (((y) << 8) | (x))
+#define ThrowGIFException(exception,message) \
+{ \
+  if (profiles != (LinkedListInfo *) NULL) \
+    profiles=DestroyLinkedList(profiles,DestroyGIFProfile); \
+  if (global_colormap != (unsigned char *) NULL) \
+    global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap); \
+  if (meta_image != (Image *) NULL) \
+    meta_image=DestroyImage(meta_image); \
+  ThrowReaderException((exception),(message)); \
+}
 
   Image
     *image,
     *meta_image;
 
-  int
-    number_extensionss=0;
+  LinkedListInfo
+    *profiles;
 
   MagickBooleanType
     status;
@@ -1020,34 +1036,27 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((count != 6) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
       (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  (void) ResetMagickMemory(buffer,0,sizeof(buffer));
+  (void) memset(buffer,0,sizeof(buffer));
   meta_image=AcquireImage(image_info);  /* metadata container */
   meta_image->page.width=ReadBlobLSBShort(image);
   meta_image->page.height=ReadBlobLSBShort(image);
   flag=(unsigned char) ReadBlobByte(image);
   background=(unsigned char) ReadBlobByte(image);
   c=(unsigned char) ReadBlobByte(image);  /* reserved */
+  profiles=(LinkedListInfo *) NULL;
   one=1;
   global_colors=one << (((size_t) flag & 0x07)+1);
   global_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
     MagickMax(global_colors,256),3UL*sizeof(*global_colormap));
   if (global_colormap == (unsigned char *) NULL)
-    {
-      meta_image=DestroyImage(meta_image);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
-  (void) ResetMagickMemory(global_colormap,0,3*MagickMax(global_colors,256)*
+    ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(global_colormap,0,3*MagickMax(global_colors,256)*
     sizeof(*global_colormap));
   if (BitSet((int) flag,0x80) != 0)
     {
       count=ReadBlob(image,(size_t) (3*global_colors),global_colormap);
       if (count != (ssize_t) (3*global_colors))
-        {
-          global_colormap=(unsigned char *) RelinquishMagickMemory(
-            global_colormap);
-          meta_image=DestroyImage(meta_image);
-          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-        }
+        ThrowGIFException(CorruptImageError,"InsufficientImageDataInFile");
     }
   duration=0;
   opacity=(-1);
@@ -1064,16 +1073,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           GIF Extension block.
         */
-
         count=ReadBlob(image,1,&c);
         if (count != 1)
-          {
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
-            meta_image=DestroyImage(meta_image);
-            ThrowReaderException(CorruptImageError,
-              "UnableToReadExtensionBlock");
-          }
+          ThrowGIFException(CorruptImageError,"UnableToReadExtensionBlock");
+        (void) memset(buffer,0,sizeof(buffer));
         switch (c)
         {
           case 0xf9:
@@ -1164,20 +1167,14 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   MagickTrue : MagickFalse;
                 iptc=LocaleNCompare((char *) buffer,"MGKIPTC0000",11) == 0 ?
                   MagickTrue : MagickFalse;
-                number_extensionss++;
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "    Reading GIF application extension");
                 info=(unsigned char *) AcquireQuantumMemory(255UL,
                   sizeof(*info));
                 if (info == (unsigned char *) NULL)
-                  {
-                    meta_image=DestroyImage(meta_image);
-                    global_colormap=(unsigned char *) RelinquishMagickMemory(
-                      global_colormap);
-                    ThrowReaderException(ResourceLimitError,
-                      "MemoryAllocationFailed");
-                  }
-                (void) ResetMagickMemory(info,0,255UL*sizeof(*info));
+                  ThrowGIFException(ResourceLimitError,
+                    "MemoryAllocationFailed");
+                (void) memset(info,0,255UL*sizeof(*info));
                 reserved_length=255;
                 for (info_length=0; ; )
                 {
@@ -1192,11 +1189,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         reserved_length,sizeof(*info));
                       if (info == (unsigned char *) NULL)
                         {
-                          meta_image=DestroyImage(meta_image);
-                          global_colormap=(unsigned char *)
-                            RelinquishMagickMemory(global_colormap);
                           info=(unsigned char *) RelinquishMagickMemory(info);
-                          ThrowReaderException(ResourceLimitError,
+                          ThrowGIFException(ResourceLimitError,
                             "MemoryAllocationFailed");
                         }
                     }
@@ -1204,10 +1198,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 profile=BlobToStringInfo(info,(size_t) info_length);
                 if (profile == (StringInfo *) NULL)
                   {
-                    meta_image=DestroyImage(meta_image);
-                    global_colormap=(unsigned char *) RelinquishMagickMemory(
-                      global_colormap);
-                    ThrowReaderException(ResourceLimitError,
+                    info=(unsigned char *) RelinquishMagickMemory(info);
+                    ThrowGIFException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
                 if (i8bim != MagickFalse)
@@ -1226,11 +1218,17 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   (void) FormatLocaleString(name,sizeof(name),"gif:%.11s",
                     buffer);
                 info=(unsigned char *) RelinquishMagickMemory(info);
-                if (magick == MagickFalse)
-                  (void) SetImageProfile(meta_image,name,profile);
-                profile=DestroyStringInfo(profile);
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "      profile name=%s",name);
+                if (magick != MagickFalse)
+                  profile=DestroyStringInfo(profile);
+                else
+                  {
+                    if (profiles == (LinkedListInfo *) NULL)
+                      profiles=NewLinkedList(0);
+                    SetStringInfoName(profile,name);
+                    (void) AppendValueToLinkedList(profiles,profile);
+                  }
               }
             break;
           }
@@ -1251,6 +1249,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
+            if (profiles != (LinkedListInfo *) NULL)
+              profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
             image=DestroyImageList(image);
             global_colormap=(unsigned char *) RelinquishMagickMemory(
               global_colormap);
@@ -1267,8 +1267,6 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     meta_image->scene=image->scene;
     (void) CloneImageProperties(image,meta_image);
     DestroyImageProperties(meta_image);
-    (void) CloneImageProfiles(image,meta_image);
-    DestroyImageProfiles(meta_image);
     image->storage_class=PseudoClass;
     image->compression=LZWCompression;
     image->columns=ReadBlobLSBShort(image);
@@ -1284,22 +1282,12 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->ticks_per_second=100;
     image->matte=opacity >= 0 ? MagickTrue : MagickFalse;
     if ((image->columns == 0) || (image->rows == 0))
-      {
-        global_colormap=(unsigned char *) RelinquishMagickMemory(
-          global_colormap);
-        meta_image=DestroyImage(meta_image);
-        ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
-      }
+      ThrowGIFException(CorruptImageError,"NegativeOrZeroImageSize");
     /*
       Inititialize colormap.
     */
     if (AcquireImageColormap(image,image->colors) == MagickFalse)
-      {
-        global_colormap=(unsigned char *) RelinquishMagickMemory(
-          global_colormap);
-        meta_image=DestroyImage(meta_image);
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      }
+      ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
     if (BitSet((int) flag,0x80) == 0)
       {
         /*
@@ -1331,23 +1319,14 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         colormap=(unsigned char *) AcquireQuantumMemory((size_t)
           MagickMax(local_colors,256),3UL*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
-          {
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
-            meta_image=DestroyImage(meta_image);
-            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-          }
-        (void) ResetMagickMemory(colormap,0,3*MagickMax(local_colors,256)*
+          ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) memset(colormap,0,3*MagickMax(local_colors,256)*
           sizeof(*colormap));
         count=ReadBlob(image,(3*local_colors)*sizeof(*colormap),colormap);
         if (count != (ssize_t) (3*local_colors))
           {
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
             colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-            meta_image=DestroyImage(meta_image);
-            ThrowReaderException(CorruptImageError,
-              "InsufficientImageDataInFile");
+            ThrowGIFException(CorruptImageError,"InsufficientImageDataInFile");
           }
         p=colormap;
         for (i=0; i < (ssize_t) image->colors; i++)
@@ -1374,6 +1353,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status=SetImageExtent(image,image->columns,image->rows);
     if (status == MagickFalse)
       {
+        if (profiles != (LinkedListInfo *) NULL)
+          profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
         global_colormap=(unsigned char *) RelinquishMagickMemory(
           global_colormap);
         meta_image=DestroyImage(meta_image);
@@ -1389,11 +1370,23 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       status=DecodeImage(image,opacity);
     InheritException(exception,&image->exception);
     if ((image_info->ping == MagickFalse) && (status == MagickFalse))
+      ThrowGIFException(CorruptImageError,"CorruptImage");
+    if (profiles != (LinkedListInfo *) NULL)
       {
-        global_colormap=(unsigned char *) RelinquishMagickMemory(
-          global_colormap);
-        meta_image=DestroyImage(meta_image);
-        ThrowReaderException(CorruptImageError,"CorruptImage");
+        StringInfo
+          *profile;
+
+        /*
+          Set image profiles.
+        */
+        ResetLinkedListIterator(profiles);
+        profile=(StringInfo *) GetNextValueInLinkedList(profiles);
+        while (profile != (StringInfo *) NULL)
+        {
+          (void) SetImageProfile(image,GetStringInfoName(profile),profile);
+          profile=(StringInfo *) GetNextValueInLinkedList(profiles);
+        }
+        profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
       }
     duration+=image->delay*image->iterations;
     if (image_info->number_scenes != 0)
@@ -1406,6 +1399,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
   }
   image->duration=duration;
+  if (profiles != (LinkedListInfo *) NULL)
+    profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
   meta_image=DestroyImage(meta_image);
   global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap);
   if ((image->columns == 0) || (image->rows == 0))
