@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -450,11 +450,15 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   if (pixels == (unsigned char *) NULL)
     return((unsigned char *) NULL);
   *extent=row_bytes*image->rows*sizeof(*pixels);
-  (void) ResetMagickMemory(pixels,0,*extent);
+  (void) memset(pixels,0,*extent);
   scanline=(unsigned char *) AcquireQuantumMemory(row_bytes,2*
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
-    return((unsigned char *) NULL);
+    {
+      pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+      return((unsigned char *) NULL);
+    }
+  (void) memset(scanline,0,2*row_bytes*sizeof(*scanline));
   if (bytes_per_line < 8)
     {
       /*
@@ -480,7 +484,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
               image->filename);
             break;
           }
-        (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+        (void) memcpy(q,p,(size_t) number_pixels);
       }
       scanline=(unsigned char *) RelinquishMagickMemory(scanline);
       return(pixels);
@@ -494,8 +498,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     if (bytes_per_line > 200)
       scanline_length=ReadBlobMSBShort(blob);
     else
-      scanline_length=1UL*ReadBlobByte(blob);
-    if (scanline_length >= row_bytes)
+      scanline_length=(size_t) ReadBlobByte(blob);
+    if ((scanline_length >= row_bytes) || (scanline_length == 0))
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
           CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
@@ -515,7 +519,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           number_pixels=length*bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
           if ((q-pixels+number_pixels) <= *extent)
-            (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+            (void) memcpy(q,p,(size_t) number_pixels);
           q+=number_pixels;
           j+=(ssize_t) (length*bytes_per_pixel+1);
         }
@@ -527,7 +531,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           for (i=0; i < (ssize_t) length; i++)
           {
             if ((q-pixels+number_pixels) <= *extent)
-              (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+              (void) memcpy(q,p,(size_t) number_pixels);
             q+=number_pixels;
           }
           j+=(ssize_t) bytes_per_pixel+1;
@@ -598,7 +602,7 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
     Pack scanline.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(scanline != (unsigned char *) NULL);
@@ -796,8 +800,9 @@ static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
   rectangle->left=(short) ReadBlobMSBShort(image);
   rectangle->bottom=(short) ReadBlobMSBShort(image);
   rectangle->right=(short) ReadBlobMSBShort(image);
-  if ((EOFBlob(image) != MagickFalse) || (rectangle->left > rectangle->right) ||
-      (rectangle->top > rectangle->bottom))
+  if ((EOFBlob(image) != MagickFalse) ||
+      ((rectangle->bottom-rectangle->top) <= 0) ||
+      ((rectangle->right-rectangle->left) <= 0))
     return(MagickFalse);
   return(MagickTrue);
 }
@@ -805,12 +810,25 @@ static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
 static Image *ReadPICTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+#define ThrowPICTException(exception,message) \
+{ \
+  if (tile_image != (Image *) NULL) \
+    tile_image=DestroyImage(tile_image); \
+  if (read_info != (ImageInfo *) NULL) \
+    read_info=DestroyImageInfo(read_info); \
+  ThrowReaderException((exception),(message)); \
+}
+
   char
     geometry[MaxTextExtent],
     header_ole[4];
 
   Image
-    *image;
+    *image,
+    *tile_image;
+
+  ImageInfo
+    *read_info;
 
   IndexPacket
     index;
@@ -859,12 +877,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -875,6 +893,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   /*
     Read PICT header.
   */
+  read_info=(ImageInfo *) NULL;
+  tile_image=(Image *) NULL;
   pixmap.bits_per_pixel=0;
   pixmap.component_count=0;
   /*
@@ -891,31 +911,31 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         break;
   (void) ReadBlobMSBShort(image);  /* skip picture size */
   if (ReadRectangle(image,&frame) == MagickFalse)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   while ((c=ReadBlobByte(image)) == 0) ;
   if (c != 0x11)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  version=ReadBlobByte(image);
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+  version=(ssize_t) ReadBlobByte(image);
   if (version == 2)
     {
       c=ReadBlobByte(image);
       if (c != 0xff)
-        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        ThrowPICTException(CorruptImageError,"ImproperImageHeader");
     }
   else
     if (version != 1)
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) ||
       (frame.bottom < 0) || (frame.left >= frame.right) ||
       (frame.top >= frame.bottom))
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   /*
     Create black canvas.
   */
   flags=0;
   image->depth=8;
-  image->columns=1UL*(frame.right-frame.left);
-  image->rows=1UL*(frame.bottom-frame.top);
+  image->columns=(unsigned int) (frame.right-frame.left);
+  image->rows=(unsigned int) (frame.bottom-frame.top);
   image->x_resolution=DefaultResolution;
   image->y_resolution=DefaultResolution;
   image->units=UndefinedResolution;
@@ -926,6 +946,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         return(GetFirstImageInList(image));
       }
   status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
+  status=ResetImagePixels(image,exception);
   if (status == MagickFalse)
     {
       InheritException(exception,&image->exception);
@@ -946,6 +972,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       code=ReadBlobMSBSignedShort(image);
     if (code < 0)
       break;
+    if (code == 0)
+      continue;
     if (code > 0xa1)
       {
         if (image->debug != MagickFalse)
@@ -972,18 +1000,23 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (((frame.left & 0x8000) != 0) || ((frame.top & 0x8000) != 0))
               break;
-            image->columns=1UL*(frame.right-frame.left);
-            image->rows=1UL*(frame.bottom-frame.top);
+            image->columns=(size_t) (frame.right-frame.left);
+            image->rows=(size_t) (frame.bottom-frame.top);
             status=SetImageExtent(image,image->columns,image->rows);
             if (status == MagickFalse)
               {
                 InheritException(exception,&image->exception);
                 return(DestroyImageList(image));
               }
-            (void) SetImageBackgroundColor(image);
+            status=ResetImagePixels(image,exception);
+            if (status == MagickFalse)
+              {
+                InheritException(exception,&image->exception);
+                return(DestroyImageList(image));
+              }
             break;
           }
           case 0x12:
@@ -1000,7 +1033,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             /*
               Skip pattern definition.
             */
-            pattern=1L*ReadBlobMSBShort(image);
+            pattern=(ssize_t) ReadBlobMSBShort(image);
             for (i=0; i < 8; i++)
               if (ReadBlobByte(image) == EOF)
                 break;
@@ -1012,23 +1045,23 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (pattern != 1)
-              ThrowReaderException(CorruptImageError,"UnknownPatternType");
+              ThrowPICTException(CorruptImageError,"UnknownPatternType");
             length=ReadBlobMSBShort(image);
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (ReadPixmap(image,&pixmap) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-            image->depth=1UL*pixmap.component_size;
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+            image->depth=(size_t) pixmap.component_size;
             image->x_resolution=1.0*pixmap.horizontal_resolution;
             image->y_resolution=1.0*pixmap.vertical_resolution;
             image->units=PixelsPerInchResolution;
             (void) ReadBlobMSBLong(image);
-            flags=1L*ReadBlobMSBShort(image);
+            flags=(ssize_t) ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
             for (i=0; i <= (ssize_t) length; i++)
               (void) ReadBlobMSBLong(image);
-            width=1UL*(frame.bottom-frame.top);
-            height=1UL*(frame.right-frame.left);
+            width=(size_t) (frame.bottom-frame.top);
+            height=(size_t) (frame.right-frame.left);
             if (pixmap.bits_per_pixel <= 8)
               length&=0x7fff;
             if (pixmap.bits_per_pixel == 16)
@@ -1042,7 +1075,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     break;
               }
             else
-              for (j=0; j < (int) height; j++)
+              for (i=0; i < (ssize_t) height; i++)
+              {
+                if (EOFBlob(image) != MagickFalse)
+                  break;
                 if (length > 200)
                   {
                     for (j=0; j < (ssize_t) ReadBlobMSBShort(image); j++)
@@ -1053,6 +1089,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   for (j=0; j < (ssize_t) ReadBlobByte(image); j++)
                     if (ReadBlobByte(image) == EOF)
                       break;
+              }
             break;
           }
           case 0x1b:
@@ -1109,15 +1146,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             unsigned char
               *pixels;
 
-            Image
-              *tile_image;
-
             /*
               Pixmap clipped by a rectangle.
             */
             bytes_per_line=0;
             if ((code != 0x9a) && (code != 0x9b))
-              bytes_per_line=1L*ReadBlobMSBShort(image);
+              bytes_per_line=(ssize_t) ReadBlobMSBShort(image);
             else
               {
                 (void) ReadBlobMSBShort(image);
@@ -1125,24 +1159,20 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 (void) ReadBlobMSBShort(image);
               }
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             /*
               Initialize tile image.
             */
-            tile_image=CloneImage(image,1UL*(frame.right-frame.left),
-              1UL*(frame.bottom-frame.top),MagickTrue,exception);
+            tile_image=CloneImage(image,(size_t) (frame.right-frame.left),
+              (size_t) (frame.bottom-frame.top),MagickTrue,exception);
             if (tile_image == (Image *) NULL)
-              return((Image *) NULL);
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if ((code == 0x9a) || (code == 0x9b) ||
                 ((bytes_per_line & 0x8000) != 0))
               {
                 if (ReadPixmap(image,&pixmap) == MagickFalse)
-                  {
-                    tile_image=DestroyImage(tile_image);
-                    ThrowReaderException(CorruptImageError,
-                      "ImproperImageHeader");
-                  }
-                tile_image->depth=1UL*pixmap.component_size;
+                  ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+                tile_image->depth=(size_t) pixmap.component_size;
                 tile_image->matte=pixmap.component_count == 4 ?
                   MagickTrue : MagickFalse;
                 tile_image->x_resolution=(double) pixmap.horizontal_resolution;
@@ -1160,16 +1190,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if ((bytes_per_line & 0x8000) != 0)
                   {
                     (void) ReadBlobMSBLong(image);
-                    flags=1L*ReadBlobMSBShort(image);
+                    flags=(ssize_t) ReadBlobMSBShort(image);
                     tile_image->colors=1UL*ReadBlobMSBShort(image)+1;
                   }
                 status=AcquireImageColormap(tile_image,tile_image->colors);
                 if (status == MagickFalse)
-                  {
-                    tile_image=DestroyImage(tile_image);
-                    ThrowReaderException(ResourceLimitError,
-                      "MemoryAllocationFailed");
-                  }
+                  ThrowPICTException(ResourceLimitError,
+                    "MemoryAllocationFailed");
                 if ((bytes_per_line & 0x8000) != 0)
                   {
                     for (i=0; i < (ssize_t) tile_image->colors; i++)
@@ -1199,15 +1226,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   }
               }
             if (ReadRectangle(image,&source) == MagickFalse)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-              }
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (ReadRectangle(image,&destination) == MagickFalse)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-              }
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             (void) ReadBlobMSBShort(image);
             if ((code == 0x91) || (code == 0x99) || (code == 0x9b))
               {
@@ -1221,16 +1242,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               }
             if ((code != 0x9a) && (code != 0x9b) &&
                 (bytes_per_line & 0x8000) == 0)
-              pixels=DecodeImage(image,tile_image,1UL*bytes_per_line,1,&extent);
+              pixels=DecodeImage(image,tile_image,(size_t) bytes_per_line,1,
+                &extent);
             else
-              pixels=DecodeImage(image,tile_image,1UL*bytes_per_line,1U*
-                pixmap.bits_per_pixel,&extent);
+              pixels=DecodeImage(image,tile_image,(unsigned int) bytes_per_line,
+                (unsigned int) pixmap.bits_per_pixel,&extent);
             if (pixels == (unsigned char *) NULL)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              }
+              ThrowPICTException(ResourceLimitError,"MemoryAllocationFailed");
             /*
               Convert PICT tile image to pixel packets.
             */
@@ -1238,10 +1256,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             for (y=0; y < (ssize_t) tile_image->rows; y++)
             {
               if (p > (pixels+extent+image->columns))
-                {
-                  tile_image=DestroyImage(tile_image);
-                  ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
-                }
+                ThrowPICTException(CorruptImageError,"NotEnoughPixelData");
               q=QueueAuthenticPixels(tile_image,0,y,tile_image->columns,1,
                 exception);
               if (q == (PixelPacket *) NULL)
@@ -1264,7 +1279,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     if (pixmap.bits_per_pixel == 16)
                       {
-                        i=(*p++);
+                        i=(ssize_t) (*p++);
                         j=(*p);
                         SetPixelRed(q,ScaleCharToQuantum(
                           (unsigned char) ((i & 0x7c) << 1)));
@@ -1278,11 +1293,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       if (tile_image->matte == MagickFalse)
                         {
                           if (p > (pixels+extent+2*image->columns))
-                            {
-                              tile_image=DestroyImage(tile_image);
-                              ThrowReaderException(CorruptImageError,
-                                "NotEnoughPixelData");
-                            }
+                            ThrowPICTException(CorruptImageError,
+                              "NotEnoughPixelData");
                           SetPixelRed(q,ScaleCharToQuantum(*p));
                           SetPixelGreen(q,ScaleCharToQuantum(
                             *(p+tile_image->columns)));
@@ -1292,11 +1304,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       else
                         {
                           if (p > (pixels+extent+3*image->columns))
-                            {
-                              tile_image=DestroyImage(tile_image);
-                              ThrowReaderException(CorruptImageError,
-                                "NotEnoughPixelData");
-                            }
+                            ThrowPICTException(CorruptImageError,
+                              "NotEnoughPixelData");
                           SetPixelAlpha(q,ScaleCharToQuantum(*p));
                           SetPixelRed(q,ScaleCharToQuantum(
                             *(p+tile_image->columns)));
@@ -1318,16 +1327,17 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   if (p < pixels)
                     break;
                 }
-              status=SetImageProgress(image,LoadImageTag,y,tile_image->rows);
+              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                tile_image->rows);
               if (status == MagickFalse)
                 break;
             }
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-            if (jpeg == MagickFalse)
+            if ((jpeg == MagickFalse) && (EOFBlob(image) == MagickFalse))
               if ((code == 0x9a) || (code == 0x9b) ||
                   ((bytes_per_line & 0x8000) != 0))
                 (void) CompositeImage(image,CopyCompositeOp,tile_image,
-                  destination.left,destination.top);
+                  (ssize_t) destination.left,(ssize_t) destination.top);
             tile_image=DestroyImage(tile_image);
             break;
           }
@@ -1347,7 +1357,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (length == 0)
               break;
             (void) ReadBlobMSBLong(image);
-            length-=4;
+            length-=MagickMin(length,4);
             if (length == 0)
               break;
             info=(unsigned char *) AcquireQuantumMemory(length,sizeof(*info));
@@ -1357,7 +1367,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (count != (ssize_t) length)
               {
                 info=(unsigned char *) RelinquishMagickMemory(info);
-                ThrowReaderException(ResourceLimitError,"UnableToReadImageData");
+                ThrowPICTException(ResourceLimitError,"UnableToReadImageData");
               }
             switch (type)
             {
@@ -1370,7 +1380,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (status == MagickFalse)
                   {
                     info=(unsigned char *) RelinquishMagickMemory(info);
-                    ThrowReaderException(ResourceLimitError,
+                    ThrowPICTException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
                 break;
@@ -1383,7 +1393,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (status == MagickFalse)
                   {
                     info=(unsigned char *) RelinquishMagickMemory(info);
-                    ThrowReaderException(ResourceLimitError,
+                    ThrowPICTException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
                 profile=DestroyStringInfo(profile);
@@ -1424,15 +1434,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       continue;
     if (code == 0x8200)
       {
+        char
+          filename[MaxTextExtent];
+
         FILE
           *file;
 
-        Image
-          *tile_image;
-
-        ImageInfo
-          *read_info;
-
         int
           unique_file;
 
@@ -1443,7 +1450,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         read_info=CloneImageInfo(image_info);
         SetImageInfoBlob(read_info,(void *) NULL,0);
         file=(FILE *) NULL;
-        unique_file=AcquireUniqueFileResource(read_info->filename);
+        unique_file=AcquireUniqueFileResource(filename);
+        (void) FormatLocaleString(read_info->filename,MaxTextExtent,"jpeg:%s",
+          filename);
         if (unique_file != -1)
           file=fdopen(unique_file,"wb");
         if ((unique_file == -1) || (file == (FILE *) NULL))
@@ -1465,7 +1474,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 (void) fclose(file);
                 (void) RelinquishUniqueFileResource(read_info->filename);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+                ThrowPICTException(CorruptImageError,"ImproperImageHeader");
               }
             for (i=0; i < 122; i++)
               if (ReadBlobByte(image) == EOF)
@@ -1481,7 +1490,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         (void) fclose(file);
         (void) close(unique_file);
         tile_image=ReadImage(read_info,exception);
-        (void) RelinquishUniqueFileResource(read_info->filename);
+        (void) RelinquishUniqueFileResource(filename);
         read_info=DestroyImageInfo(read_info);
         if (tile_image == (Image *) NULL)
           continue;
@@ -1492,8 +1501,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           MagickMax(image->columns,tile_image->columns),
           MagickMax(image->rows,tile_image->rows));
         (void) TransformImageColorspace(image,tile_image->colorspace);
-        (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
-          frame.right);
+        (void) CompositeImage(image,CopyCompositeOp,tile_image,(ssize_t)
+          frame.left,(ssize_t) frame.right);
         image->compression=tile_image->compression;
         tile_image=DestroyImage(tile_image);
         continue;
@@ -1699,9 +1708,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 65535L) || (image->rows > 65535L))
@@ -1778,12 +1787,12 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  (void) ResetMagickMemory(scanline,0,row_bytes);
-  (void) ResetMagickMemory(packed_scanline,0,(size_t) (row_bytes+MaxCount));
+  (void) memset(scanline,0,row_bytes);
+  (void) memset(packed_scanline,0,(size_t) (row_bytes+MaxCount));
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
-  (void) ResetMagickMemory(buffer,0,PictInfoSize);
+  (void) memset(buffer,0,PictInfoSize);
   (void) WriteBlob(image,PictInfoSize,buffer);
   (void) WriteBlobMSBShort(image,0);
   (void) WriteBlobMSBShort(image,(unsigned short) size_rectangle.top);
@@ -1793,7 +1802,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   (void) WriteBlobMSBShort(image,PictVersion);
   (void) WriteBlobMSBShort(image,0x02ff);  /* version #2 */
   (void) WriteBlobMSBShort(image,PictInfoOp);
-  (void) WriteBlobMSBLong(image,0xFFFE0000UL);
+  (void) WriteBlobMSBLong(image,0xFFFE0000U);
   /*
     Write full size of the file, resolution, frame bounding box, and reserved.
   */
@@ -1824,13 +1833,13 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,(unsigned short)
         (GetStringInfoLength(profile)+4));
-      (void) WriteBlobMSBLong(image,0x00000000UL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
       (void) WriteBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,4);
-      (void) WriteBlobMSBLong(image,0x00000002UL);
+      (void) WriteBlobMSBLong(image,0x00000002U);
     }
   /*
     Write crop region opcode and crop bounding box.
@@ -1873,50 +1882,50 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,PictJPEGOp);
       (void) WriteBlobMSBLong(image,(unsigned int) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,0x00010000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00010000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x40000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00400000UL);
+      (void) WriteBlobMSBLong(image,0x00010000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00010000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x40000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00400000U);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00566A70UL);
-      (void) WriteBlobMSBLong(image,0x65670000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000001UL);
-      (void) WriteBlobMSBLong(image,0x00016170UL);
-      (void) WriteBlobMSBLong(image,0x706C0000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00566A70U);
+      (void) WriteBlobMSBLong(image,0x65670000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000001U);
+      (void) WriteBlobMSBLong(image,0x00016170U);
+      (void) WriteBlobMSBLong(image,0x706C0000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) x_resolution);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) y_resolution);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x87AC0001UL);
-      (void) WriteBlobMSBLong(image,0x0B466F74UL);
-      (void) WriteBlobMSBLong(image,0x6F202D20UL);
-      (void) WriteBlobMSBLong(image,0x4A504547UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x0018FFFFUL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x87AC0001U);
+      (void) WriteBlobMSBLong(image,0x0B466F74U);
+      (void) WriteBlobMSBLong(image,0x6F202D20U);
+      (void) WriteBlobMSBLong(image,0x4A504547U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x0018FFFFU);
       (void) WriteBlob(image,length,blob);
       if ((length & 0x01) != 0)
         (void) WriteBlobByte(image,'\0');
@@ -1930,7 +1939,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     {
       (void) WriteBlobMSBShort(image,PictPixmapOp);
-      (void) WriteBlobMSBLong(image,(size_t) base_address);
+      (void) WriteBlobMSBLong(image,(unsigned int) base_address);
     }
   (void) WriteBlobMSBShort(image,(unsigned short) (row_bytes | 0x8000));
   (void) WriteBlobMSBShort(image,(unsigned short) bounds.top);
@@ -2014,7 +2023,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     if (image_info->compression == JPEGCompression)
       {
-        (void) ResetMagickMemory(scanline,0,row_bytes);
+        (void) memset(scanline,0,row_bytes);
         for (y=0; y < (ssize_t) image->rows; y++)
           count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
             packed_scanline);
