@@ -315,7 +315,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read SGI raster header.
   */
-  (void) ResetMagickMemory(&iris_info,0,sizeof(iris_info));
+  (void) memset(&iris_info,0,sizeof(iris_info));
   iris_info.magic=ReadBlobMSBShort(image);
   do
   {
@@ -380,6 +380,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         InheritException(exception,&image->exception);
         return(DestroyImageList(image));
       }
+    (void) SetImageBackgroundColor(image);
     /*
       Allocate SGI pixels.
     */
@@ -393,6 +394,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (pixel_info == (MemoryInfo *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
+    (void) memset(pixels,0,iris_info.columns*iris_info.rows*4*
+      bytes_per_pixel*sizeof(*pixels));
     if ((int) iris_info.storage != 0x01)
       {
         unsigned char
@@ -953,7 +956,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
       Initialize SGI raster file header.
     */
     (void) TransformImageColorspace(image,sRGBColorspace);
-    (void) ResetMagickMemory(&iris_info,0,sizeof(iris_info));
+    (void) memset(&iris_info,0,sizeof(iris_info));
     iris_info.magic=0x01DA;
     compression=image->compression;
     if (image_info->compression != UndefinedCompression)
@@ -1129,6 +1132,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
               runlength=(size_t *) RelinquishMagickMemory(runlength);
             if (packet_info != (MemoryInfo *) NULL)
               packet_info=RelinquishVirtualMemory(packet_info);
+            pixel_info=RelinquishVirtualMemory(pixel_info);
             ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
           }
         packets=(unsigned char *) GetVirtualMemoryBlob(packet_info);
